@@ -149,3 +149,91 @@ AIC.fitMk<-function(object,...,k=2){
 	-2*logLik(object)+np*k
 }
 	
+## S3 plot method for objects of class "fitMk"
+plot.fitMk<-function(x,...){
+	if(hasArg(signif)) signif<-list(...)$signif
+	else signif<-3
+	if(hasArg(main)) main<-list(...)$main
+	else main<-NULL
+	if(hasArg(cex.main)) cex.main<-list(...)$cex.main
+	else cex.main<-1.2
+	if(hasArg(cex.traits)) cex.traits<-list(...)$cex.traits
+	else cex.traits<-1
+	if(hasArg(cex.rates)) cex.rates<-list(...)$cex.rates
+	else cex.rates<-0.6
+	if(hasArg(show.zeros)) show.zeros<-list(...)$show.zeros
+	else show.zeros<-TRUE
+	if(hasArg(tol)) tol<-list(...)$tol
+	else tol<-1e-6
+	if(hasArg(mar)) mar<-list(...)$mar
+	else mar<-c(1.1,1.1,3.1,1.1)
+	if(hasArg(lwd)) lwd<-list(...)$lwd
+	else lwd<-1
+	Q<-matrix(NA,length(x$states),length(x$states))
+    	Q[]<-c(0,x$rates)[x$index.matrix+1]
+	diag(Q)<-0
+	spacer<-0.1
+	plot.new()
+	par(mar=mar)
+	xylim<-c(-1.2,1.2)
+	plot.window(xlim=xylim,ylim=xylim,asp=1)
+	if(!is.null(main)) title(main=main,cex.main=cex.main)
+	nstates<-length(x$states)
+	step<-360/nstates
+	angles<-seq(0,360-step,by=step)/180*pi
+	if(nstates==2) angles<-angles+pi/2
+	v.x<-cos(angles)
+	v.y<-sin(angles)
+	for(i in 1:nstates) for(j in 1:nstates)
+		if(if(!isSymmetric(Q)) i!=j else i>j){
+			dx<-v.x[j]-v.x[i]
+			dy<-v.y[j]-v.y[i]
+			slope<-abs(dy/dx)
+			shift.x<-0.02*sin(atan(dy/dx))*sign(j-i)*if(dy/dx>0) 1 else -1
+			shift.y<-0.02*cos(atan(dy/dx))*sign(j-i)*if(dy/dx>0) -1 else 1
+			s<-c(v.x[i]+spacer*cos(atan(slope))*sign(dx)+
+				if(isSymmetric(Q)) 0 else shift.x,
+				v.y[i]+spacer*sin(atan(slope))*sign(dy)+
+				if(isSymmetric(Q)) 0 else shift.y)
+			e<-c(v.x[j]+spacer*cos(atan(slope))*sign(-dx)+
+				if(isSymmetric(Q)) 0 else shift.x,
+				v.y[j]+spacer*sin(atan(slope))*sign(-dy)+
+				if(isSymmetric(Q)) 0 else shift.y)
+			if(show.zeros||Q[i,j]>tol){
+				if(abs(diff(c(i,j)))==1||abs(diff(c(i,j)))==(nstates-1))
+					text(mean(c(s[1],e[1]))+1.5*shift.x,
+						mean(c(s[2],e[2]))+1.5*shift.y,
+						round(Q[i,j],signif),cex=cex.rates,
+						srt=atan(dy/dx)*180/pi)
+				else
+					text(mean(c(s[1],e[1]))+0.3*diff(c(s[1],e[1]))+
+						1.5*shift.x,
+						mean(c(s[2],e[2]))+0.3*diff(c(s[2],e[2]))+
+						1.5*shift.y,
+						round(Q[i,j],signif),cex=cex.rates,
+						srt=atan(dy/dx)*180/pi)
+				arrows(s[1],s[2],e[1],e[2],length=0.05,
+					code=if(isSymmetric(Q)) 3 else 2,lwd=lwd)
+			}
+		}
+	text(v.x,v.y,x$states,cex=cex.traits,
+		col=make.transparent("black",0.7))
+}
+
+## S3 plot method for objects resulting from fitDiscrete
+plot.gfit<-function(x,...){
+	if("mkn"%in%class(x$lik)==FALSE){
+		stop("Sorry. No plot method presently available for objects of this type.")
+	} else {
+		obj<-list()
+		QQ<-.Qmatrix.from.gfit(x)
+		obj$states<-colnames(QQ)
+		m<-length(obj$states)
+		obj$index.matrix<-matrix(NA,m,m)
+		k<-m*(m-1)
+		obj$index.matrix[col(obj$index.matrix)!=row(obj$index.matrix)]<-1:k
+		obj$rates<-QQ[sapply(1:k,function(x,y) which(x==y),obj$index.matrix)]
+		class(obj)<-"fitMk"
+		plot(obj,...)
+	}
+}
